from enum import Enum


class CrewmateColour(Enum):
    BLACK = 1
    BLUE = 2
    BROWN = 3
    CYAN = 4
    DARKGREEN = 5
    DARK_GREEN = 5
    LIME = 6
    ORANGE = 7
    PINK = 8
    PURPLE = 9
    RED = 10
    WHITE = 11
    YELLOW = 12
    RANDOM = 13


class _BaseEnum(Enum):
    def __str__(self) -> str:
        return self.value


class AllEndpoints(_BaseEnum):
    ADIOS = "/adios"
    BATMANSLAP = "/batmanslap"
    CARREVERSE = "/carreverse"
    CHANGEMYMIND = "/changemymind"
    DISTRACTEDBF = "/distractedbf"
    DOCKOFSHAME = "/dockofshame"
    DRIP = "/drip"
    EJECTED = "/ejected"
    EMERGENCYMEETING = "/emergencymeeting"
    FIRSTTIME = "firsttime"
    GRAVE = "/grave"
    HEAVEN = "/heaven"
    IAMSPEED = "/iamspeed"
    ICANMILKYOU = "/icanmilkyou"
    NPC = "/npc"
    PEEPOSIGN = "/peeposign"
    RANKCARD = "/rankcard"
    STONKS = "/stonks"
    TABLEFLIP = "/tableflip"
    WATER = "/water"
    WIDE = "/wide"
    WOLVERINE = "/wolverine"
    WOMANYELLINGATCAT = "/womanyellingatcat"


class TextEndpoints(_BaseEnum):
    CARREVERSE = "/carreverse"
    CHANGEMYMIND = "/changemymind"
    EMERGENCYMEETING = "/emergencymeeting"
    NPC = "/npc"
    PEEPOSIGN = "/peeposign"
    WATER = "/water"


class UserEndpoints(_BaseEnum):
    ADIOS = "/adios"
    DOCKOFSHAME = "/dockofshame"
    DRIP = "/drip"
    FIRSTTIME = "/firsttime"
    GRAVE = "/grave"
    IAMSPEED = "/iamspeed"
    ICANMILKYOU = "/icanmilkyou"
    HEAVEN = "/heaven"
    TABLEFLIP = "/tableflip"
    WOLVERINE = "/wolverine"


class OtherEndpoints(_BaseEnum):
    BATMANSLAP = "/batmanslap"
    DISTRACTEDBF = "/distractedbf"
    EJECTED = "/ejected"
    RANKCARD = "/rankcard"
    STONKS = "/stonks"
    WIDE = "/wide"
    WOMANYELLINGATCAT = "/womanyellingatcat"


class ImageEndpoints(_BaseEnum):
    WIDE = "/wide"


class Badges(_BaseEnum):
    BOOST = "boost"
    BRAVERY = "bravery"
    BRILLIANCE = "brilliance"
    BUG_HUNTER = "bughunter"
    DEVELOPER = "developer"
    EARLY_SUPPORTER = "earlysupporter"
    EVENTS = "events"
    NITRO = "nitro"
    PARTNER = "partner"
    STAFF = "staff"



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Tuple

if TYPE_CHECKING:
    from aiohttp import ClientResponse


__all__: Tuple[str, ...] = (
    "VacEfronException",
    "BadRequest",
    "NotFound",
    "InternalServerError",
    "Forbidden",
    "HTTPException",
)


class VacEfronException(Exception):
    pass


class BadRequest(VacEfronException):
    pass


class NotFound(VacEfronException):
    pass


class Forbidden(VacEfronException):
    pass


class InternalServerError(VacEfronException):
    pass


class HTTPException(VacEfronException):
    def __init__(self, response: ClientResponse, message: Any):
        self.response = response
        self.status = response.status
        self.message = message



from __future__ import annotations
from typing import TYPE_CHECKING, Literal, Optional, Tuple, Union, overload

from random import choice as choose

from aiohttp import ClientSession

from . import utils as _utils
from .models.rank import Rankcard
from .errors import *
from .http import HTTPClient as _HTTPClient
from .enums import ImageEndpoints, TextEndpoints, UserEndpoints, CrewmateColour

if TYPE_CHECKING:
    from aiohttp import ClientSession
    from .models.image import Image

__all__: Tuple[str, ...] = ("Client",)


class Client:
    """Represents a Client for the alexflipnote API.

    Parameters
    ----------
    session: Optional[:class:`aiohtttp.ClientSession`]
        The session to use for the HTTPClient. If not provided, a new one will be created.
    """

    __slots__: Tuple[str, ...] = ("__http",)

    def __init__(self, *, session: Optional[ClientSession] = None) -> None:
        self.__http = _HTTPClient(self, session=session)

    # Json/URL

    @overload
    async def discord_server(self, *, creator: Literal[False] = ...) -> str:
        ...

    @overload
    async def discord_server(self, *, creator: Literal[True] = ...) -> Tuple[str, str]:
        ...

    async def discord_server(self, *, creator: bool = False) -> Union[str, Tuple[str, str]]:
        """Returns an invite to VacEfron's discord server.

        Parameters
        ----------
        creator: Optional[:class:`bool`]
            Whether to return the invite to the creator of the bot.

        Returns
        -------
        Union[:class:`str`, :class:`Tuple[:class:`str`, :class:`str`]`]
            The invite to the VacEfron's server
                if creator is False, otherwise a tuple with an invite to the
                    alexflipnote server and the creator of this wrapper.
        """
        data = await self.__http.base_page()
        support_server = data["discord_server"]
        if creator is True:
            return support_server, "https://discord.gg/yCzcfju"

        return support_server

    async def readme(self) -> str:
        """Returns the readme of the API.

        Returns
        -------
        :class:`str`
            The readme of the API.
        """
        data = await self.__http.base_page()
        return data["readme"]

    # User

    async def adios(self, user: str) -> Image:
        """Returns an image of meme: "adios".

        Parameters
        ----------
        user: :class:`str`
            The avatar of the user to use on the image.

        Returns
        -------
        :class:`Image`
            An Image object representing the generated image.
        """
        response = await self.__http.with_users(UserEndpoints.ADIOS, user)
        return self.__http._handle_image(response)

    async def dock_of_shame(self, user: str) -> Image:
        """Returns an image of meme: "dock of shame".

        Parameters
        ----------
        user: :class:`str`
            The user avatar to use in the image.

        Returns
        -------
        :class:`Image`
            An Image object representing the generated image.
        """
        response = await self.__http.with_users(UserEndpoints.DOCKOFSHAME, user)
        return self.__http._handle_image(response)

    async def drip(self, user: str) -> Image:
        """Returns an image of meme: "drip".

        Parameters
        ----------
        user: :class:`str`
            The user avatar to use in the image.

        Returns
        -------
        :class:`Image`
            An Image object representing the generated image.
        """
        response = await self.__http.with_users(UserEndpoints.DRIP, user)
        return self.__http._handle_image(response)

    async def wolverine(self, user: str) -> Image:
        """Returns an image of meme: "wolverine".

        Parameters
        ----------
        user: :class:`str`
            The user avatar to use in the image.

        Returns
        -------
        :class:`Image`
            An Image object representing the generated image.
        """
        response = await self.__http.with_users(UserEndpoints.WOLVERINE, user)
        return self.__http._handle_image(response)

    async def first_time(self, user: str) -> Image:
        """Returns an image of meme: "first time".

        Parameters
        ----------
        user: :class:`str`
            The user avatar to use in the image.

        Returns
        -------
        :class:`Image`
            An Image object representing the generated image.
        """
        response = await self.__http.with_users(UserEndpoints.FIRSTTIME, user)
        return self.__http._handle_image(response)

    async def grave(self, user: str) -> Image:
        """Returns an image of meme: "grave".

        Parameters
        ----------
        user: :class:`str`
            The user avatar to use in the image.

        Returns
        -------
        :class:`Image`
            An Image object representing the generated image.
        """
        response = await self.__http.with_users(UserEndpoints.GRAVE, user)
        return self.__http._handle_image(response)

    async def iam_speed(self, user: str) -> Image:
        """Returns an image of meme: "i am speed".

        Parameters
        ----------
        user: :class:`str`
            The user avatar to use in the image.

        Returns
        -------
        :class:`Image`
            An Image object representing the generated image.
        """
        response = await self.__http.with_users(UserEndpoints.IAMSPEED, user)
        return self.__http._handle_image(response)

    async def i_can_milk_you(self, user: str, user2: Optional[str] = None) -> Image:
        """Returns an image of meme: "i can milk you".

        Parameters
        ----------
        user: :class:`str`
            The user avatar to use in the image.
        user2: Optional[:class:`str`]
            The user avatar to use in the image. This is optional.

        Returns
        -------
        :class:`Image`
            An Image object representing the generated image.
        """
        response = await self.__http.with_users(UserEndpoints.ICANMILKYOU, user, user2)
        return self.__http._handle_image(response)

    async def heaven(self, user: str) -> Image:
        """Returns an image of meme: "heaven".

        Parameters
        ----------
        user: :class:`str`
            The user avatar to use in the image.

        Returns
        -------
        :class:`Image`
            An Image object representing the generated image.
        """
        response = await self.__http.with_users(UserEndpoints.HEAVEN, user)
        return self.__http._handle_image(response)

    async def table_flip(self, user: str) -> Image:
        """Returns an image of meme: "table flip".

        Parameters
        ----------
        user: :class:`str`
            The user avatar to use in the image.

        Returns
        -------
        :class:`Image`
            An Image object representing the generated image.
        """
        response = await self.__http.with_users(UserEndpoints.TABLEFLIP, user)
        return self.__http._handle_image(response)

    # Text

    async def car_reverse(self, text: str) -> Image:
        """Returns an image of meme: "car reverse"

        Parameters
        ----------
        text: :class:`str`
            The text to use in the image.

        Returns
        -------
        :class:`Image`
            An Image object representing the generated image.
        """
        response = await self.__http.with_text(TextEndpoints.CARREVERSE, text)
        return self.__http._handle_image(response)

    async def change_my_mind(self, text: str) -> Image:
        """Returns an image of meme: "change my mind"

        Parameters
        ----------
        text: :class:`str`
            The text to use in the image.

        Returns
        -------
        :class:`Image`
            An Image object representing the generated image.
        """
        response = await self.__http.with_text(TextEndpoints.CHANGEMYMIND, text)
        return self.__http._handle_image(response)

    async def emergency_meeting(self, text: str) -> Image:
        """Returns an image of meme: "emergency meeting"

        Parameters
        ----------
        text: :class:`str`
            The text to use in the image.

        Returns
        -------
        :class:`Image`
            An Image object representing the generated image.
        """
        response = await self.__http.with_text(TextEndpoints.EMERGENCYMEETING, text)
        return self.__http._handle_image(response)

    async def npc(self, text: str, text2: str) -> Image:
        """Returns an image of meme: "npc"

        Parameters
        ----------
        text: :class:`str`
            The text to use in the image.
        text2: :class:`str`
            The text to use in the image.

        Returns
        -------
        :class:`Image`
            An Image object representing the generated image.
        """
        response = await self.__http.with_text(TextEndpoints.NPC, text, text2)
        return self.__http._handle_image(response)

    async def water(self, text: str) -> Image:
        """Returns an image of text: "water"

        Parameters
        ----------
        text: :class:`str`
            The text to use in the image.

        Returns
        -------
        :class:`Image`
            An Image object representing the generated image.
        """
        response = await self.__http.with_text(TextEndpoints.WATER, text)
        return self.__http._handle_image(response)

    async def peepo_sign(self, text: str) -> Image:
        """Returns an image of text: "peepo sign"

        Parameters
        ----------
        text: :class:`str`
            The text to use in the image.

        Returns
        -------
        :class:`Image`
            An Image object representing the generated image.
        """
        response = await self.__http.with_text(TextEndpoints.PEEPOSIGN, text)
        return self.__http._handle_image(response)

    # Image

    async def wide(self, image: str) -> Image:
        """Returns an image of meme: "wide"

        Parameters
        ----------
        image: :class:`str`
            The image to use.

        Returns
        -------
        :class: `Image`
            An Image object representing the generated image.
        """
        response = await self.__http.with_image(ImageEndpoints.WIDE, image)
        return self.__http._handle_image(response)

    # Other

    async def batman_slap(
        self, text: str, text2: str, *, batman: Optional[str] = None, robin: Optional[str] = None
    ) -> Image:
        """Returns an image of meme: "batman slap"

        Parameters
        ----------
        text: :class:`str`
            The text to use in the image.
        text2: Optional[:class:`str`]
            The text to use in the image. This is optional.
        batman: Optional[:class:`str`]
            The user avatar to use in the image. This is optional.
        robin: Optional[:class:`str`]
            The user avatar to use in the image. This is optional.

        Returns
        -------
        :class:`Image`
            An Image object representing the generated image.
        """
        extras = {}
        if batman:
            extras["batman"] = batman
        if robin:
            extras["robin"] = robin
        response = await self.__http.batman_slap(text1=text, text2=text2, **extras)
        return self.__http._handle_image(response)

    async def distracted_bf(self, *, boyfriend: str, girlfriend: str, woman: str) -> Image:
        """Returns an image of meme: "distracted bf"

        Parameters
        ----------
        boyfriend: :class:`str`
            The user avatar to use in the image.
        girlfriend: :class:`str`
            The user avatar to use in the image.
        woman: :class:`str`
            The user avatar to use in the image.
        """
        response = await self.__http.distracted_bf(boyfriend=boyfriend, girlfriend=girlfriend, woman=woman)
        return self.__http._handle_image(response)

    async def stonks(self, user: str, *, stonks: bool = True) -> Image:
        """Returns an image of meme: "stonks"

        Parameters
        ----------
        user: :class:`str`
            The user avatar to use in the image.
        stonks: :class:`bool`
            Whether it's stonks or not stonks. Defaults to True.
        """
        response = await self.__http.stonks(user=user, notStonks=not stonks)
        return self.__http._handle_image(response)

    async def ejected(
        self,
        name: str,
        crewmate: Union[str, int, CrewmateColour] = CrewmateColour.RED,
        impostor: bool = False,
        imposter: bool = False,
    ) -> Image:
        """Returns an image of meme: "ejected"

        Parameters
        ----------
        name: :class:`str`
            The name to use in the image.
        crewmate: Union[:class:`str`, :class:`int`, :class:`CrewmateColour`]
            The crewmate colour to use in the image.
            E,g CrewmateColour.RED

            Defaults to CrewmateColour.RED.
        impostor: :class:`bool`
            Whether the crewmate is an impostor. Defaults to False.
        imposter: :class:`bool`
            Alias for impostor.
        """
        impostor = impostor or imposter
        val = crewmate  # type: ignore
        if val is not CrewmateColour:
            try:
                if isinstance(crewmate, str):
                    val = CrewmateColour[str(crewmate.upper())]
                elif isinstance(crewmate, int):
                    val = CrewmateColour(int(crewmate))
            except (KeyError, ValueError):
                pass
            else:
                val = choose(list(CrewmateColour))

        val: CrewmateColour
        response = await self.__http.ejected(name=name, impostor=imposter, crewmate=val.value)
        return self.__http._handle_image(response)

    async def woman_yelling_at_cat(self, woman: str, cat: str) -> Image:
        """Returns an image of meme: "Woman Yelling at Cat"

        Parameters
        ----------
        woman: :class:`str`
            The user avatar to use in the image.
        cat: :class:`str`
            The user avatar to use in the image.
        """
        response = await self.__http.woman_yelling_at_woman(woman=woman, cat=cat)
        return self.__http._handle_image(response)

    async def rankcard(
        self,
        rankcard: Rankcard,
        **kwargs,
    ) -> Rankcard:
        """Validates and returns the Rankcard object with state.

        Parameters
        ----------
        rankcard: :class:`Rankcard`
            The Rankcard object to validate.
        """
        if not isinstance(rankcard, Rankcard):
            base = f"'rankcard=' must be a Rankcard object. got {rankcard.__class__.__name__!r} instead."
            if kwargs:
                valid_kwargs, invalid_kwargs = _utils._convert_rankcard_kwargs(**kwargs)
                join_kwargs = ", ".join(f"{k}={v!r}" for k, v in valid_kwargs.items())
                converted_signature = f"Rankcard({join_kwargs})"

                base += (
                    "\nI see that you are using keyword arguments, that is not supported anymore since version 2.0! "
                    "You must use construct a Rankcard object and pass it as the first argument now. "
                    "See the documentation for more information but here is an example of how to convert your code:\n"
                    f"I have converted your keyword arguments to a Rankcard object for you: {converted_signature}."
                )
                if invalid_kwargs:
                    invalid_kwargs = ", ".join(f"{k} -> {v!r}" for k, v in invalid_kwargs.items())
                    base += f"\n\nI did find some invalid keyword arguments, here is what I converted them to and the ones I couldn't convert: {invalid_kwargs}."

                support_server = await self.discord_server(creator=True)
                base += "\nIf you are still having issues, please join the {support_server} and ask for help there."
                base += "\n\n!!! This detailed error will be removed in the next version !!! please update your code."

            raise TypeError(base)

        rankcard = rankcard.copy()
        return await self.__http.rankcard(rankcard)

    async def close(self) -> None:
        await self.__http.close()

    # Aliases
    womanyellingatcat = woman_yelling_at_cat
    icanmilkyou = i_can_milk_you
    iamspeed = iam_speed
    emergencymeeting = emergency_meeting
    changemymind = change_my_mind
    carreverse = car_reverse
    dockofshame = dock_of_shame
    distractedbf = distracted_bf
    batmanslap = batman_slap
    firsttime = first_time
    rank_card = rankcard
    peeposign = peepo_sign



from __future__ import annotations
from typing import Coroutine, Optional, Tuple, ClassVar, Union, Any, TYPE_CHECKING

import aiohttp
import urllib.parse

from .errors import *
from .models.image import Image
from .enums import OtherEndpoints

if TYPE_CHECKING:
    from aiohttp import ClientResponse
    from typing_extensions import Unpack

    from .client import Client
    from .models import Rankcard
    from .enums import TextEndpoints, UserEndpoints, ImageEndpoints
    from .types.http import (
        Raw,
        BatmanSlap as BatmanSlapData,
        DistractedBf as DistractedBfData,
        Ejected as EjectedData,
        Stonks as StonksData,
        WomanYellingAtCat as WomanYellingAtCatData,
    )

    Response = Coroutine[Any, Any, ClientResponse]
    ValidEndpoint = Union[TextEndpoints, UserEndpoints, OtherEndpoints, ImageEndpoints]


class HTTPClient:
    BASE_URL: ClassVar[str] = "https://vacefron.nl/api"

    __slots__: Tuple[str, ...] = (
        "_client",
        "__session",
    )

    def __init__(self, client: Client, session: Optional[aiohttp.ClientSession] = None) -> None:
        self._client: Client = client
        self.__session: Optional[aiohttp.ClientSession] = session

    async def initiate_session(self) -> None:
        if not self.__session or self.__session.closed:
            self.__session = aiohttp.ClientSession()

    async def request(self, endpoint: Optional[ValidEndpoint] = None, **params: Any) -> ClientResponse:
        url = f"{self.BASE_URL}"
        return_json = False
        if endpoint:
            url = f"{self.BASE_URL}{str(endpoint)}"
        else:
            return_json = True
        if params:
            encoded_param = urllib.parse.urlencode(params, quote_via=urllib.parse.quote)  # type: ignore
            url += f"?{str(encoded_param)}"

        await self.initiate_session()
        if not self.__session:
            raise RuntimeError(
                "Session is not initialized. This should never happen."
            )  # just to get rid of the linter warning.

        async with self.__session.get(url) as response:
            if response.status == 200:
                if return_json:
                    return await response.json()
                return response

            elif response.status == 400:
                raise BadRequest(await response.text())
            elif response.status == 403:
                raise Forbidden(await response.text())
            elif response.status == 404:
                raise NotFound(await response.text())
            elif response.status == 500:
                raise InternalServerError(await response.text())
            else:
                raise HTTPException(response, await response.text())

    def _handle_image(self, response: ClientResponse) -> Image:
        return Image.from_url(str(response.url), self.__session)  # type: ignore

    def with_users(self, endpoint: UserEndpoints, *users: Union[str, None]) -> Response:
        data = {}
        if len(users) == 1 and users[0] is not None:
            data["user"] = users[0]
        else:
            for i, user in enumerate(users, start=1):
                if user is not None:
                    data[f"user{i}"] = user

        return self.request(endpoint, **data)

    def with_text(self, endpoint: TextEndpoints, *texts: Union[str, None]) -> Response:
        data = {}
        if len(texts) == 1:
            data["text"] = texts[0]
        else:
            for i, text in enumerate(texts, start=1):
                if text:
                    data[f"text{i}"] = text

        return self.request(endpoint, **data)

    def with_image(self, endpoint: ImageEndpoints, *images: Union[str, None]) -> Response:
        data = {}
        if len(images) == 1:
            data["image"] = images[0]
        else:
            for i, image in enumerate(images, start=1):
                if image:
                    data[f"image{i}"] = image

        return self.request(endpoint, **data)

    async def rankcard(self, obj: Rankcard) -> Rankcard:
        res = await self.request(OtherEndpoints.RANKCARD, **obj.to_dict())
        obj._image = self._handle_image(res)
        return obj

    def batman_slap(self, **options: Unpack[BatmanSlapData]) -> Response:
        return self.request(OtherEndpoints.BATMANSLAP, **options)

    def distracted_bf(self, **options: Unpack[DistractedBfData]) -> Response:
        return self.request(OtherEndpoints.DISTRACTEDBF, **options)

    def stonks(self, **options: Unpack[StonksData]) -> Response:
        return self.request(OtherEndpoints.STONKS, **options)

    def woman_yelling_at_woman(self, **options: Unpack[WomanYellingAtCatData]) -> Response:
        return self.request(OtherEndpoints.WOMANYELLINGATCAT, **options)

    def ejected(self, **options: Unpack[EjectedData]) -> Response:
        return self.request(OtherEndpoints.EJECTED, **options)

    def base_page(self) -> Coroutine[Any, Any, Raw]:
        return self.request()  # type: ignore

    async def close(self) -> None:

        if self.__session and not self.__session.closed:
            await self.__session.close()

        self.__session = None



from re import MULTILINE as re_MULTILINE, search as re_search

from setuptools import setup  # type: ignore


readme = ""
with open("README.md") as f:
    readme = f.read()

version = ""
with open("vacefron/__init__.py") as f:
    version: str = re_search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re_MULTILINE).group(1)  # type: ignore

if not version:
    raise RuntimeError("version is not set")

requirements = ["aiohttp"]
_GITHUB_URL: str = "https://github.com/Soheab/vacefron.py"
_CLASSIFIERS = [
    "Development Status :: 5 - Production/Stable",
    "Framework :: aiohttp",
    "License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)",
    "Natural Language :: English",
    "Operating System :: OS Independent",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "Programming Language :: Python :: 3.10",
    "Topic :: Internet",
    "Topic :: Software Development :: Libraries",
    "Topic :: Software Development :: Libraries :: Python Modules",
    "Topic :: Utilities",
    "Typing :: Typed",
]
_KEYWORDS = [
    "vacefron",
    "discord",
    "api",
    "wrapper",
    "memes",
    "image",
    "discord.py",
    "rankcard",
    "rank",
    "card",
    "aiohttp",
    "leveling",
]
_URLS = {
        "Discord": "https://discord.gg/xJ2HRxZ",
        "Documentation": f"{_GITHUB_URL}/blob/master/docs.md",
        "Issue tracker": f"{_GITHUB_URL}/issues",
    }
setup(
    author="Soheab",
    name="vacefron.py",
    url=_GITHUB_URL,
    description="A Wrapper for vacefron.nl/api written in Python.",
    license="MPL-2.0",
    long_description=readme,
    long_description_content_type="text/markdown",
    version=version,
    packages=["vacefron"],
    download_url=f"{_GITHUB_URL}/archive/refs/tags/v{version}.tar.gz",
    install_requires=requirements,
    keywords=_KEYWORDS,
    project_urls=_URLS,
    python_requires=">=3.8",
    classifiers=_CLASSIFIERS,
)



from typing import Any, Dict, Tuple

from inspect import signature

from .models.rank import Rankcard


def _convert_rankcard_kwargs(**kwargs: Any) -> Tuple[Dict[str, Any], Dict[str, Any]]:
    rankcard_init_signature = signature(Rankcard.__init__)
    invalid_kwargs = {}
    valid_kwargs = {}
    invalid_kwargs_conversions = {
        "name": "username",
        "avatar": "avatar_url",
        "custom_background": "background",
    }
    for key, value in kwargs.items():
        if key in rankcard_init_signature.parameters:
            valid_kwargs[key] = value
        else:
            maybe_valid_key = invalid_kwargs_conversions.get(key)
            if not maybe_valid_key:
                invalid_kwargs[key] = "Invalid!"
            else:
                invalid_kwargs[key] = maybe_valid_key
                valid_kwargs[maybe_valid_key] = value

    return valid_kwargs, invalid_kwargs
